@@ -73,22 +73,24 @@ class DagsterK8sPipelineStatusException(Exception):
 
 
 WHITELISTED_TRANSIENT_K8S_STATUS_CODES = [
-    429,  # Too many requests
+    503,  # Service unavailable
     504,  # Gateway timeout
 ]
 
 
 def k8s_api_retry(
     fn,
+    max_retries,
+    timeout,
     msg_fn=lambda: "Unexpected error encountered in Kubernetes API Client.",
-    max_retries=1,
-    timeout=10,
 ):
     check.callable_param(fn, "fn")
+    check.int_param(max_retries, "max_retries")
+    check.numeric_param(timeout, "timeout")
 
-    retries = max_retries
-    while retries > 0:
-        retries -= 1
+    remaining_attempts = 1 + max_retries
+    while remaining_attempts > 0:
+        remaining_attempts -= 1
 
         try:
             return fn()
@@ -99,10 +101,10 @@ def k8s_api_retry(
             # Check if the status code is generally whitelisted
             whitelisted = status in WHITELISTED_TRANSIENT_K8S_STATUS_CODES
 
-            # If there are remaining retries, swallow the error
-            if whitelisted and retries > 0:
+            # If there are remaining attempts, swallow the error
+            if whitelisted and remaining_attempts > 0:
                 time.sleep(timeout)
-            elif whitelisted and retries == 0:
+            elif whitelisted and remaining_attempts == 0:
                 raise_from(
                     DagsterK8sAPIRetryLimitExceeded(
                         msg_fn(),
@@ -148,72 +150,67 @@ class DagsterKubernetesClient:
             time.time,
         )
 
-    def delete_job(
-        self, job_name, namespace,
+    ### Job operations ###
+
+    def wait_for_job(
+        self,
+        job_name,
+        namespace,
+        wait_timeout=DEFAULT_WAIT_TIMEOUT,
+        wait_time_between_attempts=DEFAULT_WAIT_BETWEEN_ATTEMPTS,
+        start_time=None,
     ):
-        """Delete Kubernetes Job. We also need to delete corresponding pods due to:
-        https://github.com/kubernetes-client/python/issues/234
+        """ Wait for a job to launch and be running.
 
         Args:
             job_name (str): Name of the job to wait for.
             namespace (str): Namespace in which the job is located.
+            wait_timeout (numeric, optional): Timeout after which to give up and raise exception.
+                Defaults to DEFAULT_WAIT_TIMEOUT.
+            wait_time_between_attempts (numeric, optional): Wait time between polling attempts. Defaults
+                to DEFAULT_WAIT_BETWEEN_ATTEMPTS.
+
+        Raises:
+            DagsterK8sError: Raised when wait_timeout is exceeded or an error is encountered.
         """
         check.str_param(job_name, "job_name")
         check.str_param(namespace, "namespace")
+        check.numeric_param(wait_timeout, "wait_timeout")
+        check.numeric_param(wait_time_between_attempts, "wait_time_between_attempts")
 
-        try:
-            pod_names = self.get_pod_names_for_job(job_name, namespace)
+        job = None
+        start = start_time or self.timer()
 
-            # Collect all the errors so that we can post-process before raising
-            pod_names = self.get_pod_names_for_job(job_name, namespace)
+        while not job:
+            if self.timer() - start > wait_timeout:
+                raise DagsterK8sTimeoutError(
+                    "Timed out while waiting for job {job_name}"
+                    " to launch".format(job_name=job_name)
+                )
 
-            errors = []
-            try:
-                self.batch_api.delete_namespaced_job(name=job_name, namespace=namespace)
-            except Exception as e:  # pylint: disable=broad-except
-                errors.append(e)
+            # Get all jobs in the namespace and find the matching job
+            def _get_jobs_for_namespace():
+                jobs = self.batch_api.list_namespaced_job(
+                    namespace=namespace, field_selector="metadata.name={}".format(job_name)
+                )
+                if jobs.items:
+                    check.invariant(
+                        len(jobs.items) == 1,
+                        'There should only be one k8s job with name "{}", but got multiple jobs:" {}'.format(
+                            job_name, jobs.items
+                        ),
+                    )
+                    return jobs.items[0]
+                else:
+                    return None
 
-            for pod_name in pod_names:
-                try:
-                    self.core_api.delete_namespaced_pod(name=pod_name, namespace=namespace)
-                except Exception as e:  # pylint: disable=broad-except
-                    errors.append(e)
+            job = k8s_api_retry(
+                _get_jobs_for_namespace, max_retries=3, timeout=wait_time_between_attempts
+            )
 
-            if len(errors) > 0:
-                # Raise first non-expected error. Else, raise first error.
-                for error in errors:
-                    if not (
-                        isinstance(error, kubernetes.client.rest.ApiException)
-                        and error.reason == "Not Found"
-                    ):
-                        raise error
-                raise errors[0]
-
-            return True
-        except kubernetes.client.rest.ApiException as e:
-            if e.reason == "Not Found":
-                return False
-            raise e
-
-    def get_pod_names_for_job(self, job_name, namespace):
-        """Get pod names that corresponds to job name
-
-        Args:
-            job_name (str): Name of the job to wait for.
-            namespace (str): Namespace in which the job is located.
-        """
-        check.str_param(job_name, "job_name")
-        check.str_param(namespace, "namespace")
-
-        pods = self.core_api.list_namespaced_pod(
-            label_selector="job-name=={}".format(job_name), namespace=namespace
-        )
-
-        pod_names = []
-        for item in pods.items:
-            pod_names.append(item.metadata.name)
-
-        return pod_names
+            if not job:
+                self.logger('Job "{job_name}" not yet launched, waiting'.format(job_name=job_name))
+                self.sleeper(wait_time_between_attempts)
 
     def wait_for_job_success(
         self,
@@ -246,27 +243,16 @@ class DagsterKubernetesClient:
         check.numeric_param(wait_time_between_attempts, "wait_time_between_attempts")
         check.int_param(num_pods_to_wait_for, "num_pods_to_wait_for")
 
-        job = None
         start = self.timer()
 
-        # Wait for job to launch
-        while not job:
-            if self.timer() - start > wait_timeout:
-                raise DagsterK8sTimeoutError(
-                    "Timed out while waiting for job {job_name}"
-                    " to launch".format(job_name=job_name)
-                )
-
-            # Get all jobs in the namespace and find the matching job
-            def _get_jobs_for_namespace():
-                jobs = self.batch_api.list_namespaced_job(namespace=namespace)
-                return next((j for j in jobs.items if j.metadata.name == job_name), None)
-
-            job = k8s_api_retry(_get_jobs_for_namespace, max_retries=3)
-
-            if not job:
-                self.logger('Job "{job_name}" not yet launched, waiting'.format(job_name=job_name))
-                self.sleeper(wait_time_between_attempts)
+        # Wait for job to be running
+        self.wait_for_job(
+            job_name,
+            namespace,
+            wait_timeout=wait_timeout,
+            wait_time_between_attempts=wait_time_between_attempts,
+            start_time=start,
+        )
 
         # Wait for the job status to be completed. We check the status every
         # wait_time_between_attempts seconds
@@ -285,7 +271,9 @@ class DagsterKubernetesClient:
                 job = self.batch_api.read_namespaced_job_status(job_name, namespace=namespace)
                 return job.status
 
-            status = k8s_api_retry(_get_job_status, max_retries=3)
+            status = k8s_api_retry(
+                _get_job_status, max_retries=3, timeout=wait_time_between_attempts
+            )
 
             # status.succeeded represents the number of pods which reached phase Succeeded.
             if status.succeeded == num_pods_to_wait_for:
@@ -310,69 +298,72 @@ class DagsterKubernetesClient:
 
             self.sleeper(wait_time_between_attempts)
 
-    def retrieve_pod_logs(self, pod_name, namespace):
-        """Retrieves the raw pod logs for the pod named `pod_name` from Kubernetes.
-
-        Args:
-            pod_name (str): The name of the pod from which to retrieve logs.
-            namespace (str): The namespace of the pod.
-
-        Returns:
-            str: The raw logs retrieved from the pod.
-        """
-        check.str_param(pod_name, "pod_name")
-        check.str_param(namespace, "namespace")
-
-        # We set _preload_content to False here to prevent the k8 python api from processing the response.
-        # If the logs happen to be JSON - it will parse in to a dict and then coerce back to a str leaving
-        # us with invalid JSON as the quotes have been switched to '
-        #
-        # https://github.com/kubernetes-client/python/issues/811
-        return six.ensure_str(
-            self.core_api.read_namespaced_pod_log(
-                name=pod_name, namespace=namespace, _preload_content=False
-            ).data
-        )
-
-    def wait_for_job(
-        self,
-        job_name,
-        namespace,
-        wait_timeout=DEFAULT_WAIT_TIMEOUT,
-        wait_time_between_attempts=DEFAULT_WAIT_BETWEEN_ATTEMPTS,
+    def delete_job(
+        self, job_name, namespace,
     ):
-        """ Wait for a job to launch and be running.
+        """Delete Kubernetes Job. We also need to delete corresponding pods due to:
+        https://github.com/kubernetes-client/python/issues/234
 
         Args:
             job_name (str): Name of the job to wait for.
             namespace (str): Namespace in which the job is located.
-            wait_timeout (numeric, optional): Timeout after which to give up and raise exception.
-                Defaults to DEFAULT_WAIT_TIMEOUT.
-            wait_time_between_attempts (numeric, optional): Wait time between polling attempts. Defaults
-                to DEFAULT_WAIT_BETWEEN_ATTEMPTS.
-
-        Raises:
-            DagsterK8sError: Raised when wait_timeout is exceeded or an error is encountered.
         """
         check.str_param(job_name, "job_name")
         check.str_param(namespace, "namespace")
-        check.numeric_param(wait_timeout, "wait_timeout")
-        check.numeric_param(wait_time_between_attempts, "wait_time_between_attempts")
 
-        job = None
+        try:
+            pod_names = self.get_pod_names_in_job(job_name, namespace)
 
-        start = self.timer()
-        # Ensure we found the job that we launched
-        while not job:
-            if self.timer() - start > wait_timeout:
-                raise DagsterK8sError("Timed out while waiting for job to launch")
+            # Collect all the errors so that we can post-process before raising
+            pod_names = self.get_pod_names_in_job(job_name, namespace)
 
-            jobs = self.batch_api.list_namespaced_job(namespace=namespace)
-            job = next((j for j in jobs.items if j.metadata.name == job_name), None)
+            errors = []
+            try:
+                self.batch_api.delete_namespaced_job(name=job_name, namespace=namespace)
+            except Exception as e:  # pylint: disable=broad-except
+                errors.append(e)
 
-            if not job:
-                self.logger('Job "{job_name}" not yet launched, waiting'.format(job_name=job_name))
-                self.sleeper(wait_time_between_attempts)
+            for pod_name in pod_names:
+                try:
+                    self.core_api.delete_namespaced_pod(name=pod_name, namespace=namespace)
+                except Exception as e:  # pylint: disable=broad-except
+                    errors.append(e)
+
+            if len(errors) > 0:
+                # Raise first non-expected error. Else, raise first error.
+                for error in errors:
+                    if not (
+                        isinstance(error, kubernetes.client.rest.ApiException)
+                        and error.reason == "Not Found"
+                    ):
+                        raise error
+                raise errors[0]
+
+            return True
+        except kubernetes.client.rest.ApiException as e:
+            if e.reason == "Not Found":
+                return False
+            raise e
+
+    ### Pod operations ###
+
+    def get_pod_names_in_job(self, job_name, namespace):
+        """Get the names of pods launched by the job ``job_name``.
+
+        Args:
+            job_name (str): Name of the job to inspect.
+            namespace (str): Namespace in which the job is located.
+
+        Returns:
+            List[str]: List of all pod names that have been launched by the job ``job_name``.
+        """
+        check.str_param(job_name, "job_name")
+        check.str_param(namespace, "namespace")
+
+        pods = self.core_api.list_namespaced_pod(
+            namespace=namespace, label_selector="job-name={}".format(job_name)
+        ).items
+        return [p.metadata.name for p in pods]
 
     def wait_for_pod(
         self,
@@ -493,20 +484,26 @@ class DagsterKubernetesClient:
             else:
                 raise DagsterK8sError("Should not get here, unknown pod state")
 
-    def get_pod_names_in_job(self, job_name, namespace):
-        """Get the names of pods launched by the job ``job_name``.
+    def retrieve_pod_logs(self, pod_name, namespace):
+        """Retrieves the raw pod logs for the pod named `pod_name` from Kubernetes.
 
         Args:
-            job_name (str): Name of the job to inspect.
-            namespace (str): Namespace in which the job is located.
+            pod_name (str): The name of the pod from which to retrieve logs.
+            namespace (str): The namespace of the pod.
 
         Returns:
-            List[str]: List of all pod names that have been launched by the job ``job_name``.
+            str: The raw logs retrieved from the pod.
         """
-        check.str_param(job_name, "job_name")
+        check.str_param(pod_name, "pod_name")
         check.str_param(namespace, "namespace")
 
-        pods = self.core_api.list_namespaced_pod(
-            namespace=namespace, label_selector="job-name={}".format(job_name)
-        ).items
-        return [p.metadata.name for p in pods]
+        # We set _preload_content to False here to prevent the k8 python api from processing the response.
+        # If the logs happen to be JSON - it will parse in to a dict and then coerce back to a str leaving
+        # us with invalid JSON as the quotes have been switched to '
+        #
+        # https://github.com/kubernetes-client/python/issues/811
+        return six.ensure_str(
+            self.core_api.read_namespaced_pod_log(
+                name=pod_name, namespace=namespace, _preload_content=False
+            ).data
+        )
